unit ComFort;    // Visual Part of Computational Server

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI, StdCtrls, ExtCtrls,
  ComPas, Buttons;

//   ------- System Tray stuff :  move Application's icon from TaskBar to SysTray and back

Const TrayMessage = WM_USER+1;     // Don't use it for another aims !!!
Const Title = 'Computational Server';

Type
  TFormOnSystemTray = class(TForm)
    procedure AfterPause(Sender:TObject);
    procedure FormDestroy(Sender:TObject);
  private
    FPause:TTimer;
    Nim:TNotifyIconData;
  protected
    PresentOnTray:boolean;
    Procedure HookMinimize(Var Msg:TMessage); message WM_SYSCOMMAND;
    Procedure HookMaximize(var Msg : TMessage); message TrayMessage;
  public
    Procedure OnTray(yes:boolean);              // work with System Tray :  Put/Remove
    Procedure OnTrayAfterPause(Pause:integer);  // work with System Tray :  Put after Pause
end;

//   ------- Visual part of Computational Server :

  Tf_Server = class(TFormOnSystemTray)          // if You dislike it, use TForm instead!
    Panel1: TPanel;
    Memo1: TMemo;
    CheckBox1: TCheckBox;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpeedButton1Click(Sender: TObject);
  private
    Procedure MessFromThread(var m:TFortranMessage);  message WM_User;
  end;

Procedure Protocol(s:string);

var
  f_Server: Tf_Server;

implementation

{$R *.DFM}

Procedure Protocol(s:string);
Const Limit=200;
begin
  With f_Server do
    if CheckBox1.Checked or (s[1]<>'*') then
    begin
      if Memo1.Lines.Count>Limit then
        Repeat Memo1.Lines.Delete(1) Until Memo1.Lines.Count < (Limit div 2);
      Memo1.Lines.Add(TimeToStr(Time)+' : '+Trim(s));
    end;
end;

procedure Tf_Server.FormCreate(Sender: TObject);
// var hMenuHandle : HMENU;
begin
//  hMenuHandle := GetSystemMenu(Handle, FALSE);     // "Close" button will be disabled
//  if (hMenuHandle <> 0) then DeleteMenu(hMenuHandle, SC_CLOSE, MF_BYCOMMAND);
  Caption:=Title;
  Protocol('Computational Server started from  '+Application.Exename);
  ChDir(ExtractFilePath(Application.ExeName));      // Current Dir = where CompuServ.EXE lives
  if Sender is TFormOnSystemTray then (Sender as TFormOnSystemTray).OnTrayAfterPause(100);
end;

procedure Tf_Server.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
  if Sender is TFormOnSystemTray then (Sender as TFormOnSystemTray).OnTray(false);
end;

Procedure Tf_Server.MessFromThread(var m:TFortranMessage);  // message WM_User from Computational Thread
begin
  if Assigned(Server) then Server.MessFromThread(m);        // redirected to Computational Server
end;

procedure Tf_Server.SpeedButton1Click(Sender: TObject);
begin
  if not Assigned(Server) then Exit;
  if not Assigned(Server.LinkToThread) then Exit;
  Server.LinkToThread^.cf_Job:=0;
end;


//  ----------------  TFormOnSystemTray :

Procedure TFormOnSystemTray.OnTray(yes:boolean);  // Put on System Tray (yes/no)
Begin
  if yes and PresentOnTray then Exit;
  PresentOnTray:=yes;
  With Nim do     // fill parameter area for Shell_NotifyIcon
  Begin cbSize:=SizeOf(Nim);
    Wnd:=Handle;
    uID:=1;
    if Assigned(Icon) then hicon:=Icon.Handle else hicon:=0;
    uFlags:=NIF_ICON or NIF_MESSAGE or NIF_TIP;
    uCallbackMessage:=TrayMessage;
    StrPCopy(szTip,Copy(Caption,1,60));
  End;
  if yes then
  begin
    OnDestroy:=FormDestroy;
    ShowWindow(Application.Handle,SW_HIDE);   // from TaskBar
    ShowWindow(Handle,SW_HIDE);               // from Screen
    Shell_NotifyIcon(Nim_Add,@Nim);
  end else
  begin
    OnDestroy:=nil;
    ShowWindow(Application.Handle,SW_SHOWNORMAL);
    ShowWindow(Handle,SW_SHOWNORMAL);
    Shell_NotifyIcon(Nim_Delete,@Nim);
  end;
End;

procedure TFormOnSystemTray.FormDestroy(Sender: TObject);
begin
    Shell_NotifyIcon(Nim_Delete,@Nim);
end;

Procedure TFormOnSystemTray.OnTrayAfterPause(Pause:integer);  // Put on System Tray after given Pause
begin
  FPause:=TTimer.Create(Self);
  FPause.Interval:=Pause;
  FPause.OnTimer:=AfterPause;
  FPause.Enabled:=true;
end;

procedure TFormOnSystemTray.AfterPause(Sender: TObject);
begin
  FPause.Free;
  FPause:=nil;
  OnTray(true);
end;

Procedure TFormOnSystemTray.HookMinimize(Var Msg:TMessage); // message WM_SYSCOMMAND;
Begin  Case Msg.WParam of
         SC_MINIMIZE : OnTray(true);
         SC_CLOSE    : Close;
          else         inherited;
       End;
End;

Procedure TFormOnSystemTray.HookMaximize(var Msg : TMessage); // message TrayMessage;
Var p:tpoint;
begin Case Msg.LParam OF // What button pressed
    WM_LBUTTONUP,WM_LBUTTONDBLCLK: OnTray(false);
    WM_RBUTTONUP: Begin GetCursorPos(p); // Mouse Coord
                    SetForegroundWindow(Handle);
                    if Assigned(PopupMenu) then  PopupMenu.Popup(p.X,p.Y);
                    PostMessage(Handle,WM_NULL,0,0);
                  end;
  End;
End;

end.
