unit ComPas;
interface

uses
  Windows, SysUtils, ComObj, ActiveX, AxCtrls,
  Classes, StdVcl, Forms, Dialogs, Messages, Math,
  CompuServ_TLB;
//  ***************   COM Server's stuff :      ****************

Const cf_LBuf=1600;                  // see ComFort.F90  : the same declarations!
Const SpecOutput='%%%<';

Type
  PLinkArea = ^TLinkArea;
  TLinkArea = record
    cf_Job:integer;                  // having changed  - see EQUIVALENCE there
    cf_MainHandle:THandle;
    cf_ConsoleStatus:integer;
    cf_BufIO:Array[0..cf_LBuf-1] of char;
  end;

  TFortranMessage = record
     msg:Cardinal;
     Unused:integer;
     P:PLinkArea;
  end;

  TCallDLL = Procedure(var a:TLinkArea); stdcall;      //   Subroutine DLLname(job)  - main entry into DLL

  TComputationalThread = class(TThread)
  private
    mess:string;
    LastWrit:string;
    Procedure SendParams(s:string);
  protected
    LibHandle:THandle;
    T0:TDateTime;
    StartProc:TCallDLL;
    Injector:TCallDLL;
    DLLFile,DLLProc:string;
    Params:TLinkArea;
    Procedure Execute; override;
  end;

Type
  TComputationalServer = class(TAutoObject, IConnectionPointContainer, IComputationalServer)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    FSinkList: TList;
    FEvents: IComputationalServerEvents;
    FLinkToThread:PLinkArea;
    FAnswerFromClient:string;
    procedure ThreadFinished(Sender:TObject);
  protected
    { Protected declarations }
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function  Get_Time: Integer; safecall;
    procedure Resume; safecall;
    procedure Suspend; safecall;
    procedure Terminate; safecall;
    function ExecuteDLL(const Path: WideString; const ParameterLine: WideString): WideString; safecall;
    procedure Set_AnswerFromClient(const Value: WideString); safecall;
    procedure Set_Injected(const Value: WideString); safecall;
  public
    procedure Initialize; override;
    procedure FireEvent(const evName: WideString; evValue: OleVariant);
    Procedure MessFromThread(var m:TFortranMessage);  // called from ComFort
    Property LinkToThread:PLinkArea read FLinkToThread;
  end;

Const Server:TComputationalServer = nil;   // correct because of Single Instance!
Const Thread:TComputationalThread = nil;   //         ---"---


implementation

Uses ComFort, ComServ;      // ComServ cannot be in the Interface Section?

Function GetFragment(var s:string):string;
var i:integer;
begin
  s:=Trim(s);  i:=Pos(' ',s);
  if i=0 then begin Result:=s; s:=''; end else
    begin Result:=Copy(s,1,i-1); Delete(s,1,i); end;
end;

Function FileTime2Sec(t:TFileTime):double;
begin  // this is a bit twisted because I wanted to avoid using Int64 type !
  Result:=0.0000001*abs(4096.0*1024.0*1024.0*LongInt(t.dwHighDateTime)+
                        2.0*LongInt(t.dwLowDateTime shr 1));
end;

// ---     TComputationalThread :

Procedure TComputationalThread.Execute;
begin
  f_Server.Caption:=Title+': '+DLLProc;
  Params.cf_job:=2;              // 1 - DLL called from old P_Client, 2-from COM-server
  mess:='';                      // normal termination
  try
    if not Terminated then StartProc(Params);
  except
    on EZeroDivide do mess:='Division by zero';
    on EDivByZero  do mess:='Division by zero';
    on EInvalidOp  do mess:='Invalid floating point operation';
    on EOverFlow   do mess:='Floating point overflow';
    on E:Exception do mess:=E.Message;
    else mess:='Unknown Exception';
  end;
  if mess<>'' then
  begin
    ShowMessage('Exception:'#13+mess);
    SendParams(mess);
    Params.cf_job:=0;            // after Math.Exception
    StartProc(Params);           // spec-call for return resources
  end;
  f_Server.Caption:=Title;
  if LibHandle>0 then FreeLibrary(LibHandle);
end;

Procedure TComputationalThread.SendParams(s:string);
var i:integer;
begin
  for i:=0 to cf_LBuf-1 do       // Send fixed-length Fortran-style string to DLL
    if i<Length(s) then Params.cf_BufIO[i]:=s[i+1] else Params.cf_BufIO[i]:=' ';
end;


//   -----------------  TComputationalServer :

Procedure TComputationalServer.Initialize;
begin
  inherited Initialize;
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoint := FConnectionPoints.CreateConnectionPoint(
      AutoFactory.EventIID, ckSingle, EventConnect)
  else FConnectionPoint := nil;
  Thread:=nil;                      // DLL is not launched yet
  Server:=Self;                     // but Server is just started
  Protocol('CompuServ  version : '+IntToStr(CompuServMajorVersion)+'.'+IntToStr(CompuServMinorVersion));
end;

procedure TComputationalServer.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IComputationalServerEvents;
  if FConnectionPoint <> nil then
     FSinkList := FConnectionPoint.SinkList;
end;

function TComputationalServer.ExecuteDLL(const Path: WideString; const ParameterLine: WideString): WideString;
begin
  if Assigned(Thread) then Result:=Thread.DLLFile+' is running yet!'
  else With Thread do
  begin
    Thread:=TComputationalThread.Create(true);       // suspended
    FreeOnTerminate:=true;
    OnTerminate:=Self.ThreadFinished;                // to transit this Event
    T0:=Time;                                        // starting time
    Params.cf_MainHandle:=f_Server.Handle;
    StartProc:=nil;                                  // start point
    DLLFile:=Path;
    DLLProc:=AnsiUpperCase(ChangeFileExt(ExtractFileName(DLLFile),''));
    LibHandle:=LoadLibrary(StrPCopy(Params.cf_BufIO,DLLFile));
    if LibHandle<32 then LibHandle:=0 else
      StartProc:=TCallDLL(GetProcAddress(LibHandle,StrPCopy(Params.cf_BufIO,DLLProc)));
    Injector:=TCallDLL(GetProcAddress(LibHandle,'Inject_Line_From_Outside'));
    if Assigned(StartProc) then  Result:='' else
    begin
      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,nil,GetLastError,0,@Params.cf_BufIO,cf_LBuf,nil);
      Result:='Cannot load library '+DLLFile+#13+Params.cf_BufIO;     // answer for Client
      Terminate;
    end;
    Protocol('*** Current running DLL : '+Path);
    if ParameterLine<>'' then Protocol('*** Parameter Line : '+ParameterLine);
    SendParams(ParameterLine);                       // string parameter (do not move up!)
    FLinkToThread:=nil;
  end;
end;

function TComputationalServer.Get_Time: Integer;
var  tdum,tk,tu:TFileTime;
  i,j:integer; b:boolean;
begin
  Result:=0;
  if not Assigned(Thread) then Exit;
  b:=Thread.Suspended;
  GetThreadTimes(Thread.Handle,tdum,tdum,tk,tu);
  i:=Round(FileTime2Sec(tk)+FileTime2Sec(tu));         // time a-la WinNT
  j:=Round(24.0*3600.0*(Time-Thread.T0));              // W98 doesn't support Thread times!
  if i<1 then i:=j else i:=Min(i,j);
  Result:=Max(i,1);                                    // to be #0
  if b then Result:=-Result;                           // negative if suspended
end;

procedure TComputationalServer.Resume;
begin
  if Assigned(Thread) and Thread.Suspended then Thread.Resume;
  Protocol('*** Resume');
end;

procedure TComputationalServer.Suspend;
begin
  if Assigned(Thread) and not Thread.Suspended then Thread.Suspend;
  Protocol('*** Suspend');
end;

procedure TComputationalServer.Terminate;
var Context:TContext;
  AbortProc:TCallDLL;
begin
  Protocol('*** Terminate');
  if not Assigned(Thread) then Exit;
  AbortProc:=TCallDLL(GetProcAddress(Thread.LibHandle,'SelfKiller'));
  if not Assigned(AbortProc) then Exit;
  if not Thread.Suspended then Thread.Suspend;
  Context.ContextFlags:=CONTEXT_CONTROL;
  GetThreadContext(Thread.Handle,Context);
  Context.EIP:=integer(@AbortProc);
  SetThreadContext(Thread.Handle,Context);
  Thread.Resume;
  Protocol('*** Termination ... done!');
end;

procedure TComputationalServer.Set_AnswerFromClient(const Value: WideString);
begin
  Protocol('*** Client answerred: '+Value);
  FAnswerFromClient:=Value;  // then its'll be transferred into cf_BufIO (module ComFort)
end;

procedure TComputationalServer.Set_Injected(const Value: WideString);
var b:boolean;
begin
  if not Assigned(Thread) or not Assigned(Thread.Injector) then Exit;
  b:=Thread.Suspended;
  if not b then Thread.Suspend;
  Protocol('*** Injected : '+Value);
  Thread.SendParams(Value);        // to Link_Area
  With Thread do Injector(Params); // to cf_InjectedLine  (module ComFort)
  if not b then Thread.Resume;     // do not touch if was Suspended yet!
end;

// -----  Main Thread's interaction with Client :

Procedure TComputationalServer.FireEvent(const evName: WideString; evValue: OleVariant);
begin
  Protocol('*** Fire Event : '+Trim(evName)+' = '+Trim(evValue));
  if Assigned(FEvents) then FEvents.OnEvent(evName,evValue);
end;

procedure TComputationalServer.ThreadFinished(Sender:TObject);
var s:string;
begin
  if not Assigned(FEvents) then Exit;
  if Assigned(Thread) then s:=Thread.LastWrit else s:='';
//  FEvents.OnTerminate;  // for a while I'd use General Event !!!
  FireEvent('Thread_finished',s);  // here is it
  Thread:=nil;                     // quite correct!
  FLinkToThread:=nil;
end;

Procedure TComputationalServer.MessFromThread(var m:TFortranMessage); // message WM_User from Thread
var s,ename:string; c:char; i,j,k:integer; {fin,}console:boolean;
begin
  FLinkToThread:=m.P;
  console:=(m.P^.cf_ConsoleStatus<0);
  s:=Trim(m.P^.cf_BufIO);
//  fin:=((Pos(SpecOutput+'DONE ',s)=1) or (Pos(SpecOutput+'ERROR ',s)=1));  // Finish(last output)
//  if fin then s:=s+'   Time: '+IntToStr(Get_Time);                         // add time-mark to User-output
  Case m.P^.cf_Job of
    2: ShowMessage(s);                   // Write(Debug,...)
    3: begin                             // Read(...)
         FAnswerFromClient:=#0;                 // clearing Client's answer
         if console then ReadLn(FAnswerFromClient) else
           FireEvent('Give_Line',Trim(Thread.LastWrit));  // remember last Output
         Thread.LastWrit:='';                             // then forget it
         s:=FAnswerFromClient;                  // Client's Answer MUST be in the AnswerFromClient !!!
         j:=Length(s);                          // "Give_Line" Request MUST get Answer!
         c:=' ';                                // AnswerFromClient[1] = #0  treated as EndFile !!!
         for i:=cf_LBuf downto 1 do             // final string - into DLL's buffer
         begin
           if i<=j then c:=s[i];
           if Ord(c)=9 then c:=' ';             // change Tabs to Blank
           m.P^.cf_BufIO[i-1]:=c;
         end;
       end;
    else
       begin                             // Write(...)
         ename:='Take_Line';                    // for usual Output
         if Pos(SpecOutput,s)=1 then
         begin                                  // split   %%%<NAME  VALUE
           ename:=AnsiUpperCase(GetFragment(s));
           Delete(ename,1,Length(SpecOutput));
           s:=Trim(s);
         end;
         Repeat
           k:=Pos(Chr(250),s);                  // split if line divided with Char(250)
           if k=0 then begin Thread.LastWrit:=s; s:=''; end
                  else begin Thread.LastWrit:=Copy(s,1,k-1); Delete(s,1,k); end;
           if console then WriteLn(s)
             else FireEvent(ename,Thread.LastWrit);  // generate Event for Client
         Until s='';
       end;
  end;         // Case cf_Job
  m.P^.cf_Job:=0;                        // clearing Job is a sign : Job done!
  FLinkToThread:=nil;
end;


initialization
  TAutoObjectFactory.Create(ComServer, TComputationalServer, Class_ComputationalServer,
    ciSingleInstance, tmSingle);
end.
