Unit SapDLG;   // taken from  system Unit "FileCtrl" and rearranged

interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, IniFiles, Buttons, ExtCtrls, StdCtrls, ComCtrls, FileCtrl,
  SapTools, Math;


Procedure Anons(m:string; mt:TMsgDlgType);
Function  Confirm(m:string; mt:TMsgDlgType):boolean;
Function  PoliteManner:string;
Function  PassWord:string;
Function  ChooseFromList(Title:string; List:TStringList; Default:string):string;
Function  ChooseFromFile(Title:string; FileName:string):string;
Function  ChooseDirectory(Title:string; InitDir: string) : string;
Function  ChooseFileR(Title:string; InitFile: string; Filter:string) : string;
Function  ChooseFileW(Title:string; InitFile: string; Filter:string) : string;
Procedure CanonizeName(var Name:string);

Const  Sexe:integer = 0;  // 0-undef, 1-M, 2-F

implementation

Type TChooseType = (ctDir,ctFileR,ctFileW);

  TPathLabel = class(TCustomLabel)
  protected
    procedure Paint; override;
  public
    constructor Create(AnOwner: TComponent); override;
  published
    property Alignment;
    property Transparent;
  end;

  TSelectDirDlg = class(TForm)
    DirList: TDirectoryListBox;
    DirEdit: TEdit;
    DriveList: TDriveComboBox;
    DirLabel: TPathLabel;
    FreeLabel: TLabel;
    FilterComboBox:TFilterComboBox;
    OKButton: TBitBtn;
    CancelButton: TBitBtn;
    NetButton: TBitBtn;
    FileList: TFileListBox;
    procedure DirListClick(Sender: TObject);
    procedure DirListChange(Sender: TObject);
    procedure FileListChange(Sender: TObject);
    procedure FileListDblClick(Sender: TObject);
    procedure DriveListChange(Sender: TObject);
    procedure NetClick(Sender: TObject);
  private
    { Private declarations }
    Ready:boolean;
    FChooseType: TChooseType;
    FFN:string;
    procedure SetDirectory(const Value: string);
    function GetDirectory: string;
  public
    { Public declarations }
    Constructor Create(Title:string; ChooseType:TChooseType; InitPath: string; Filt:string);
    property Directory: string read GetDirectory write SetDirectory;
  end;

Function TrailingSlash(s:string):string;
begin Result:=s;  if s='' then Exit;
  if s[Length(s)]<>'\' then Result:=s+'\';
end;

{ TPathLabel }

constructor TPathLabel.Create(AnOwner: TComponent);
begin   inherited Create(AnOwner);
  WordWrap := False;  AutoSize := False;  ShowAccelChar := False;
end;

procedure TPathLabel.Paint;
const Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var   Rect: TRect;  Temp: String;
begin  with Canvas do
  begin Rect := ClientRect;
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(Rect);
    end;
    Brush.Style := bsClear;
    Temp := MinimizeName(Caption, Canvas, Rect.Right - Rect.Left);
    DrawText(Canvas.Handle, PChar(Temp), Length(Temp), Rect, DT_NOPREFIX or Alignments[Alignment]);
  end;
end;

{ TSelectDirDlg }

Const L1=6; T1=40; H1=24; W1=70; GapH=10;
      WD=200; WF=150;
Constructor TSelectDirDLG.Create(Title:string; ChooseType:TChooseType; InitPath: string; Filt:string);
begin Ready:=false;
  inherited CreateNew(Application);
  Caption := Title;  FChooseType:=ChooseType;  FFN:=ExtractFileName(InitPath);
  ClientHeight := 300+H1+GapH;  ClientWidth := 3*L1+WD+WF;
  BorderStyle := bsDialog;  Position := poScreenCenter;
// -----------------------------------------
  DirEdit := TEdit.Create(Self);
  with DirEdit do
  begin Parent:=Self; TabOrder:=0;
    SetBounds(L1,L1, WD+WF+L1,H1);
    Text:=InitPath; Visible:=(FChooseType=ctFileW);
  end;
  DirLabel := TPathLabel.Create(Self);
  with DirLabel do
  begin Parent := Self; BoundsRect := DirEdit.BoundsRect;
    Visible:=(FChooseType<>ctFileW);
  end;
  FilterComboBox := TFilterComboBox.Create(Self);
  with FilterComboBox do
  begin Parent:=Self; TabOrder:=0;
    SetBounds(2*L1+WD,T1, WF,H1);
    Visible:=(Pos('|',Filt)>0);  OnChange:=DirListChange;
  end;
  FreeLabel := TLabel.Create(Self);
  with FreeLabel do
  begin Parent := Self;  Visible:=not FilterComboBox.Visible;
    BoundsRect := FilterComboBox.BoundsRect;
  end;
  NetClick(nil);                            // to create initial DriveList
  DirList := TDirectoryListBox.Create(Self);
  with DirList do
  begin Parent:=Self; TabOrder:=1; TabStop:=True; ItemHeight:=17;
    IntegralHeight:=True; OnChange:=DirListChange;  OnClick:=DirListClick;
    SetBounds(L1, T1+H1+GapH, WD,Self.ClientHeight-T1-2*H1-3*GapH);
  end;
  FileList := TFileListBox.Create(Self);
  with FileList do
  begin Parent:=Self; TabOrder:=6; TabStop:=True; ItemHeight:=17;
    IntegralHeight:=True; OnChange:=FileListChange;
    if FChooseType<>ctDir then OnDblClick:=FileListDblClick;
    SetBounds(2*L1+WD,T1+H1+GapH, WF,Self.ClientHeight-T1-2*H1-3*GapH);
    FileType:=[ftNormal];
    if FChooseType=ctDir then begin ItemHeight:=13; Font.Color:=clGrayText; end;
  end;
  NetButton := TBitBtn.Create(Self);
  with NetButton do
  begin Parent:=Self; TabOrder:=3;  OnClick:=NetClick;
    if Language=Russian then Caption:='...' else Caption:='Network...';
    Hint:='     Windows-'; ShowHint:=(Language=Russian);
    SetBounds(L1,T1+H1+2*GapH+DirList.Height, W1,H1);
  end;
  OKButton := TBitBtn.Create(Self);
  with OKButton do
  begin Parent := Self; TabOrder:=4; Kind:=bkOK;
    Hint:=',   '; ShowHint:=(Language=Russian);
    SetBounds(DirList.Left+WD-W1,NetButton.Top, W1,H1);
  end;
  CancelButton := TBitBtn.Create(Self);
  with CancelButton do
  begin Parent := Self;  ModalResult:=mrCancel;  Kind:=bkCancel;
    SetBounds(FileList.Left+WF-W1,NetButton.Top, W1,H1);
    TabOrder:=5; Cancel:=True; if Language=Russian then Caption:='';
  end;
  DirListChange(Self);
  Directory := InitPath;
  if FChooseType=ctDir then ActiveControl := DirList else ActiveControl:=FileList;
  if Filt='' then FilterComboBox.Filter:='*.*' else FilterComboBox.Filter:=Filt;
  Ready:=true;
end;

Procedure TSelectDirDlg.DriveListChange(Sender: TObject);
var i:int64;
begin
  if Assigned(DirList) then DirList.Drive := DriveList.Drive;
  i:=DiskFree((Ord(DriveList.Drive)-Ord('A')+1) mod 32); if i>0 then i:=i shr 10 else i:=0;
  FreeLabel.Caption:=IntToStr(i);
  Case Language of
    English: FreeLabel.Caption:='   Free: '+FreeLabel.Caption+' kBytes.';
    Russian: FreeLabel.Caption:='   : '+FreeLabel.Caption+' kB';
  end;
  DirListChange(Sender);
end;

procedure TSelectDirDlg.DirListClick(Sender: TObject);
begin  if Assigned(DirList) then DirList.OpenCurrent;  end;

procedure TSelectDirDlg.DirListChange(Sender: TObject);
begin
  if not Assigned(DirList) then Exit;
  if not Assigned(FileList) then Exit;
  if not Assigned(DirLabel) then Exit;
  if not Assigned(DirEdit) then Exit;
  DirLabel.Caption := TrailingSlash(DirList.Directory);
  FileList.Directory := DirList.Directory;
  FileList.ItemIndex:=-1;
  FileList.Mask:=FilterComboBox.Mask;
  With DirEdit do
  begin Text := DirLabel.Caption;  if FChooseType<>ctDir then Text:=Text+FFN;
    SelectAll;
    if Ready and Visible then SetFocus;
  end;
end;

procedure TSelectDirDlg.FileListChange(Sender: TObject);
begin  if FChooseType=ctDir then begin FileList.ItemIndex:=-1; {Exit;} end;
  if FileList.FileName<>'' then DirLabel.Caption := FileList.FileName;
  FileList.Directory := DirList.Directory;
  With DirEdit do
  begin Text := DirLabel.Caption;
    SelectAll;
    if Ready and Visible then SetFocus;
  end;
  if (Directory<>'') and (FileList.ItemIndex>=0) then Case Language of
    English: FreeLabel.Caption:='   Size: '+IntToStr(SizeOfFile(Directory))+' bytes.';
    Russian: FreeLabel.Caption:='   : '+IntToStr(SizeOfFile(Directory))+' .';
  end;
end;

procedure TSelectDirDlg.FileListDblClick(Sender: TObject);
begin  FileListChange(Sender);  ModalResult:=mrOK; end;

procedure TSelectDirDlg.SetDirectory(const Value: string);
var
  Temp: string;
begin if Value='' then Exit;
  Temp:=ExpandFileName(Value);
  DriveList.Drive := Temp[1];
  if DirectoryExists(Temp) then Temp:=Temp+'\' else Temp := ExtractFilePath(Temp);
  try
    DirList.Directory := Copy(Temp, 1, Length(Temp) - 1);
  except
    on EInOutError do begin GetDir(0,Temp); DriveList.Drive:=Temp[1]; DirList.Directory:=Temp; end;
  end;
end;

function TSelectDirDlg.GetDirectory: string;
begin  Result := DirLabel.Caption;  end;

procedure TSelectDirDlg.NetClick(Sender: TObject);
begin  if Assigned(Sender) then  WNetConnectionDialog(Handle, WNTYPE_DRIVE);
  DriveList.Free;   DriveList := TDriveComboBox.Create(Self);
  with DriveList do
  begin Parent:=Self;  TabOrder:=2;  OnChange:=DriveListChange;  TextCase:=tcUpperCase;
    SetBounds(L1,T1, WD,H1);
  end;
end;

function ChooseFileDir(Title:string; ChooseType:TChooseType; InitPath: string; Filter:string) : string;
var D: TSelectDirDlg; h,p:string;
begin  h:=GetCurrentDir; p:=InitPath;
  if p='' then if DirectoryExists('A:\') then p:='A:' else if DirectoryExists('B:\') then p:='B:' else p:='C:';
  D := TSelectDirDlg.Create(Title,ChooseType,p,Filter);
  try
    if Screen.PixelsPerInch <> 96 then
    begin   // scale to screen res
      D.ScaleBy(Screen.PixelsPerInch, 96);
      D.FileList.ParentFont := True;
      D.Left := (Screen.Width div 2) - (D.Width div 2);
      D.Top := (Screen.Height div 2) - (D.Height div 2);
      D.FileList.Font.Color := clGrayText;
    end;
    if D.ShowModal = mrOK  then Result:={D.Directory} D.DirEdit.Text else Result:='';
    if (ChooseType<>ctDir) and (TrailingSlash(Result)=Result) then Result:='';
  finally
    D.Free;  ChDir(h);     // not change current Dir !!!
  end;
end;

function  ChooseDirectory(Title:string; InitDir: string) : string;
begin Result:=ChooseFileDir(Title,ctDir,InitDir,''); end;

Function  ChooseFileR(Title:string; InitFile: string; Filter:string) : string;
begin Result:=ChooseFileDir(Title,ctFileR,InitFile,Filter); end;

Function  ChooseFileW(Title:string; InitFile: string; Filter:string) : string;
begin Result:=ChooseFileDir(Title,ctFileW,InitFile,Filter); end;

{ -----------------------------------------------------------------------------
  Anons(m:string; mt:TMsgDlgType) -  Message M   .
         ,         .
    MT -   MessageDlg,   ,   Language.
  Confirm(---"---) -   ,   2 : Yes or No (  )
       true   "Yes"
}
Type TOwnMessageForm = class(TForm)
  procedure Press(sender:Tobject; var c:char);
  procedure Click(sender:Tobject);
  procedure Show(sender:Tobject);
end;
procedure TOwnMessageForm.Press(sender:Tobject; var c:char);
begin if Ord(c)=VK_RETURN then ModalResult:=mrOK else if Ord(c)=VK_ESCAPE then ModalResult:=mrNO; end;
procedure TOwnMessageForm.Click(sender:Tobject);
begin ModalResult:=mrOK; end;
procedure TOwnMessageForm.Show(sender:Tobject);
begin if ActiveControl is TComboBox then With ActiveControl as TComboBox do if Text='' then DroppedDown:=true; end;

Function ChooseFromList(Title:string; List:TStringList; Default:string):string;
var F:TOwnMessageForm; L:TLabel; CB:TComboBox;  i,j:integer; s,s1:string;
begin s:=Title;  Result:='';
  for i:=0 to List.Count-1 do
  begin s1:=List[i];
    j:=Pos('=',s1); if j>1 then s1:=Copy(s1,1,j-1);
    if Length(s1)>Length(s) then s:=s1;
  end;
  F:=TOwnMessageForm(CreateMessageDialog(s+'  ',mtConfirmation,[]));
  With F do
  begin Color:=clAqua;  Caption:=Title;   Height:=Height-40;
    OnKeyPress:=Press;  OnShow:=Show; L:=TLabel(Components[1]);
    for i:=0 to ComponentCount-1 do TLabel(Components[i]).OnClick:=Click;
    CB:=TComboBox.Create(F); CB.Parent:=L.Parent; ActiveControl:=CB;
    CB.OnKeyPress:=Press; CB.OnDblClick:=Click;  CB.Text:=Default;
    CB.SetBounds(L.Left,L.Top,L.Width,L.Height);  CB.Items:=List;
    for i:=1 to CB.Items.Count do      //    Name=Value   Name !!!
      begin s:=CB.Items[i-1]; j:=Pos('=',s); if j>1 then CB.Items[i-1]:=Copy(s,1,j-1); end;
    Hint:='   '#13+'   ,'#13+'   <Enter>';
    ShowHint:=true;
    ShowModal;
    if ModalResult=mrOK then Result:=CB.Text else Result:='';
    CB.Free; CB:=nil;
  end;
  F.Free;
end;

Function ChooseFromFile(Title:string; FileName:string):string;
var L:TStringList;
begin L:=TStringList.Create; Result:='';
  try
    L.LoadFromFile(FileName);
    Result:=ChooseFromList(Title,L,'');
  finally
    L.Free;
  end;
end;

Procedure Anons(m:string; mt:TMsgDlgType);
var i:integer;
begin
  With TOwnMessageForm(CreateMessageDialog(m,mt,[])) do
  begin Color:=clAqua;  OnKeyPress:=Press;  OnClick:=Click;
    for i:=0 to ComponentCount-1 do TLabel(Components[i]).OnClick:=Click;
    Case Language of
      English:;
      Russian: Case mt of
                 mtWarning:      Caption:=' :';
                 mtError:        Caption:=' :';
                 mtInformation:  Caption:='   :';
                 mtConfirmation: Caption:=', '+PoliteManner+' :';
                 mtCustom:       Caption:=Application.Title;
               end;
    end;
    Height:=Height-40;
    ShowModal; Free;
  end;
end;

Function Confirm(m:string; mt:TMsgDlgType):boolean;
var i:integer;
begin
  With TOwnMessageForm(CreateMessageDialog(m,mt,[mbYes,mbNo])) do
  begin Color:=clAqua;
    Case Language of
      English:;
      Russian: Case mt of
                 mtWarning:      Caption:=' :';
                 mtError:        Caption:=' :';
                 mtInformation:  Caption:='   :';
                 mtConfirmation: Caption:='   :';
                 mtCustom:       Caption:=Application.Title;
               end;
    end;
    for i:=0 to ComponentCount-1 do
      if Components[i] is TButton then Case Language of
        English:;
        Russian: if TButton(Components[i]).Name='Yes'
                    then TButton(Components[i]).Caption:=''
                    else TButton(Components[i]).Caption:='';
      end;
    Result:=(ShowModal=mrYes); Free;
  end;
end;

Function  PoliteManner:string;
Const N=6;
Const M: array[0..N-1] of string = ('','','','','','  ');
Const F: array[0..N-1] of string = ('','','','','','  ');
begin  if Sexe=2 then Result:=F[Random(N)] else Result:=M[Random(N)]; end;

Procedure CanonizeName(var Name:string);  // and SexDef
var s1,s2,s3:string; i:integer;
begin s1:=Trim(AnsiLowerCase(Name)); s2:=''; s3:='';
  i:=Pos(' ',s1); if i>0 then
  begin s2:=Trim(Copy(s1,i+1,255)); s1:=Trim(Copy(s1,1,i-1));
    i:=Pos('.',s2); if i=0 then i:=Pos(' ',s2); if i>0 then
    begin s3:=Trim(Copy(s2,i+1,255)); s2:=Trim(Copy(s2,1,i-1));
      if (s3<>'') and (s3[Length(s3)]='.') then Delete(s3,Length(s3),1);
    end;
  end;
  Name:=AnsiUpperCase(s1); if s1<>'' then s1[1]:=Name[1];
  i:=Pos('-',Name); if (i>1) and (i<Length(Name)) then s1[i+1]:=Name[i+1];
  Name:=AnsiUpperCase(s2); if s2<>'' then s2[1]:=Name[1];
  Name:=AnsiUpperCase(s3); if s3<>'' then s3[1]:=Name[1];
  Name:=s1; if s2<>'' then Name:=Name+' '+s2;
  if Length(s2)=1 then Name:=Name+'.' else Name:=Name+' ';
  if s3<>'' then Name:=Name+s3; if Length(s3)=1 then Name:=Name+'.';
  Name:=Trim(Name);
  if Length(s3)<3 then s3:='  ' else s3:=Copy(s3,Length(s3)-1,2);
  if s3[2]='' then begin Sexe:=1; Exit; end;
  if s3='' then begin Sexe:=2; Exit; end;
  if Length(s2)<2 then s2:='  ' else Delete(s2,1,Length(s2)-1);
  if (s2='') or (s2='') then begin Sexe:=2; Exit; end;
  if Length(s1)>1 then
  begin Delete(s1,1,Length(s1)-2);
    if (s1='') or (s1='') or (s1='') then begin Sexe:=2; Exit; end;
    if (s1[2]='') or (s1[2]='') or (s1[2]='') then begin Sexe:=1; Exit; end;
  end;
  if Sexe>0 then Exit;    // maybe set yet !!!
  s1:=Application.Title;
  Application.Title:='  : ';
  if Confirm(',   '+Name+'   -   ?',mtCustom) then Sexe:=2 else Sexe:=1;
  Application.Title:=s1;
end;

Function PassWord:string;
var F:TOwnMessageForm; E:TEdit;
begin F:=TOwnMessageForm.CreateNew(Application);
  F.BorderStyle:=bsDialog; F.BorderIcons:=[];
  F.ClientHeight:=20; F.ClientWidth:=80; F.Position:=poScreenCenter;
  E:=TEdit.Create(F); E.Parent:=F; E.Align:=alClient; E.PasswordChar:='*';
  if Language=Russian then F.Caption:=' ?' else F.Caption:='Password ?';
  E.OnKeyPress:=F.Press;
  F.ShowModal;
  if F.ModalResult=mrOK then Result:=E.Text else Result:='';  F.Free;
end;

end.
