unit SapNedit;  //  TRNumEdit, TINumEdit  components
                //  TAnimatedButton       component
interface

uses Windows, Messages, SysUtils, Classes, Graphics, Buttons,
     Controls, Forms, Dialogs,  StdCtrls, ExtCtrls, ImgList,
     Math, SAPTools;

Type
(*   TRNumEdit,TINumEdit - clone of TEdit for input only numbers
          and validation them according ValueCheck property
          Value = current number just input in Edit.Text
          Valid = true if  ValueMin <= Value <= ValueMax
          ValueCheck allows to put on/off  one or both these bounds
          ColorUsual = Edit Field Color when valid Value
          ColorError = Edit Field Color when not valid Value
          Caption is a Label, associated with Edit Field
          CaptionPosition defines location of Label relatively to Field
*)

  TValueCheck = (vcNone,vcMin,vcMax,vcBoth);
  TCaptionPosition = (cpLeft,cpRight,cpTop,cpBottom,cpNone);

  TGenericNumEdit = class(TCustomEdit)
  private
    FLabel:TLabel;
    FCaptionPosition:TCaptionPosition;
    FMinVal,FMaxVal,FCurVal:TReal;
    FIntCurVal:TInt;
    FValid:boolean;
    FValueCheck:TValueCheck;
    FColorError,FColorUsual:TColor;
    Procedure SetCurVal(v:TReal);
    Procedure SetMinVal(v:TReal);
    Procedure SetMaxVal(v:TReal);
    Procedure SetValid(v:boolean);
    Procedure Changed(Sender:TObject);
    Procedure SetCaptionPosition(p:TCaptionPosition);
    Procedure SetCaption(s:string);
    Function  GetCaption:string;
    Procedure CMVisibleChanged(var m:TMessage); message CM_VISIBLECHANGED;
    Procedure CMEnabledChanged(var m:TMessage); message CM_ENABLEDCHANGED;
    Procedure FOnMyLabelClick(Sender:TObject);
  protected
    Procedure KeyPress(var Key:Char); override;
  public
    Constructor Create(AOwner:TComponent); override;
    Destructor  Destroy; override;
    Procedure DefaultHandler(var Message); override;
  published
    Property Align;
    Property Enabled;
    Property Font;
    Property Height;
    Property Width;
    Property HelpContext;
    Property ParentFont;
    Property TabOrder;
    Property Tag;
    Property Visible;
    Property OnEnter;
    Property OnExit;
    Property OnClick;
    Property OnDblClick;
    Property Caption:string read GetCaption Write SetCaption;
    Property CaptionPosition:TCaptionPosition read FCaptionPosition Write SetCaptionPosition;
    Property ValueMax: TReal read FMaxVal write SetMaxVal;
    Property ValueMin: TReal read FMinVal write SetMinVal;
    Property ValueCheck:TValueCheck read FValueCheck write FValueCheck;
    Property Valid: boolean read FValid  write SetValid default true;
    Property ColorError: Tcolor read FColorError write FColorError;
    Property ColorUsual: Tcolor read FColorUsual write FColorUsual;
  end                                ;

  TRNumEdit = class(TGenericNumEdit)
  published
    Property Value: TReal read FCurVal write SetCurVal;
  end                                ;

  TINumEdit = class(TGenericNumEdit)
  private
    Procedure SetIntCurVal(v:TInt);
  published
    Property Value: TInt read FIntCurVal write SetIntCurVal;
  end                                ;


  TAnimatedButton = class(TSpeedButton)
  private
    FBitMapList:TList;
    FTimer:TTimer;
    FCurrentImage:integer;
    FThread:TThread;
    FHelpContext:THelpContext;
    Procedure TickTimer(Sender:TObject);
    Procedure CMVisibleChanged(var m:TMessage); message CM_VISIBLECHANGED;
  public
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    Procedure DefaultHandler(var Message); override;
    Function Activate(fn:string; freq:integer):boolean;
    Function Active:boolean;
    Procedure Deactivate;
    Property AssociatedThread:TThread read FThread write FThread;
  published
    Property HelpContext:THelpContext read FHelpContext write FHelpContext;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('CompAS', [TRNumEdit,TINumEdit,TAnimatedButton]);
end;

Constructor TGenericNumEdit.Create(AOwner:TComponent);
var i:integer;
begin
  inherited Create(AOwner);
  Parent:=(AOwner as TWinControl);
  if Self is TRNumEdit then Width:=60 else Width:=40;
  ColorUsual:=clWindow;
  ColorError:=clRed;
  ShowHint:=true;
  Text:=' '; Valid:=true;
  OnChange:=Changed;
  OnExit:=Changed;
  ValueCheck:=vcBoth;
  FLabel:=nil;
  FCaptionPosition:=cpLeft;
  if AOwner is TForm then With AOwner as TForm do
    for i:=0 to ComponentCount-1 do
      if Components[i] is TButton then TButton(Components[i]).Default:=false;
end;

Destructor TGenericNumEdit.Destroy;
begin
  if Assigned(FLabel) then FLabel.Free;
  inherited Destroy;
end;

Procedure TGenericNumEdit.KeyPress(var Key:Char);
var wc:TWinControl;
  I, StartIndex: Integer;
  List: TList;
begin
  if Key<>Chr(VK_Return) then Exit;
  Key:=Chr(VK_TAB);
  if not (Parent is TWinControl) then Exit;
  List := TList.Create;
  try
    (Parent as TWinControl).GetTabOrderList(List);
    wc:=nil;
    if List.Count > 0 then
    begin
      StartIndex := List.IndexOf(Self);
      if StartIndex = -1 then StartIndex := 0;
      I := StartIndex;
      repeat Inc(I);
        if I = List.Count then I := 0;
        wc := List[I];
        if wc.CanFocus and wc.TabStop and (wc is TGenericNumEdit) then Break;
        wc:=nil;
      until I = StartIndex;
    end;
  finally
    List.Free;
  end;
  if Assigned(wc) then wc.SetFocus;
end;

Procedure TGenericNumEdit.CMVisibleChanged(var m:TMessage);
begin
  if Assigned(FLabel) and not (csDesigning in ComponentState)
    then FLabel.Visible:=Self.Visible;
  inherited;
end;

Procedure TGenericNumEdit.CMEnabledChanged(var m:TMessage);
begin
  if Assigned(FLabel) then FLabel.Enabled:=Self.Enabled;
  inherited;
end;

Function TGenericNumEdit.GetCaption:string;
begin
  if Assigned(FLabel) then Result:=FLabel.Caption else Result:='';
end;

Procedure TGenericNumEdit.SetCaption(s:string);
Const Gap=3;
begin
  FLabel.Free;  FLabel:=nil;  if FCaptionPosition=cpNone then Exit;
  FLabel:=TLabel.Create(Self);
  FLabel.Parent:=Parent;
  FLabel.OnClick:=FOnMyLabelClick;
  FLabel.Visible:=false;
  FLabel.Width:=FLabel.Canvas.TextWidth(s);
  FLabel.Top:=((Height-FLabel.Height) div 2) + Top;
  FLabel.Left:=((Width-FLabel.Width) div 2) + Left;
  Case FCaptionPosition of
    cpLeft  : FLabel.Left:=Max(0,Left-FLabel.Width-Gap);
    cpRight : FLabel.Left:=Left+Width+Gap;
    cpTop   : FLabel.Top:=Max(0,Top-FLabel.Height-Gap);
    cpBottom: FLabel.Top:=Top+Height+Gap;
  end;
  FLabel.Caption:=s;
  FLabel.Hint:=Self.Hint; FLabel.ShowHint:=Self.ShowHint;
  FLabel.Visible:=Self.Visible;
end;

Procedure TGenericNumEdit.FOnMyLabelClick(Sender:TObject);
begin if Assigned(OnClick) then OnClick(Self); end;

Procedure TGenericNumEdit.SetCaptionPosition(p:TCaptionPosition);
begin
  if Align=alNone then FCaptionPosition:=p else FCaptionPosition:=cpNone;
  Caption:=GetCaption;  // ! - Repaint Label on new Position
end;

Procedure TGenericNumEdit.DefaultHandler(var Message);
begin
  Case TMessage(Message).Msg of
    WM_Size,WM_Move,CM_FontChanged: Caption:=GetCaption;  // ! - Repaint Label on new Position
  end;
  inherited;
end;

Procedure TGenericNumEdit.SetValid(v:boolean);
begin  FValid:=v;
  if v then Color:=ColorUsual else
      begin Color:=ColorError;
        if not (csDesigning in ComponentState)
           and not (csLoading in ComponentState) then Beep;
      end;
end;

Procedure TGenericNumEdit.SetCurVal(v:TReal);
begin FCurval:=v; FIntCurVal:=Round(v);
  if Self is TINumEdit  // Changed will occur automatically!
    then Text:=IntToStr(FIntCurval)
    else Text:=RealToStr(FCurval,-1);
end;

Procedure TGenericNumEdit.SetMinVal(v:TReal);
begin
  if Self is TINumEdit then FMinVal:=Int(v) else FMinVal:=v;
  Changed(Self);
end;

Procedure TGenericNumEdit.SetMaxVal(v:TReal);
begin
  if Self is TINumEdit then FMaxVal:=Int(v) else FMaxVal:=v;
  Changed(Self);
end;

Procedure TGenericNumEdit.Changed(Sender:TObject);
var i:integer; s:string;
begin
  s:=Trim(Text); if s='' then s:='0';
  i:=Pos(',',s); if i>0 then s[i]:='.';
  if Pos('.',s)=Length(s) then s:=s+'0';  { Val gives error for "0."  ! }
  Val(s,FCurVal,i);  FIntCurVal:=Round(FCurval);
  Valid:=(i=0);
  if ((ValueCheck=vcMin) or (ValueCheck=vcBoth)) and (FCurVal<FMinVal) then Valid:=false;
  if ((ValueCheck=vcMax) or (ValueCheck=vcBoth)) and (FCurVal>FMaxVal) then Valid:=false;
  s:=Hint;
  if Language=Russian then s:=' ' else s:='Sensible value';
  if Self is TINumEdit then
  begin
    if Language=Russian then s:=s+' ()' else s:=s+' (integer)';
    if Valid and (Pos('.',s)>0) then (Self as TINumEdit).Value:=FIntCurVal;
  end;
  if (ValueCheck=vcMin) or (ValueCheck=vcBoth)
    then s:=RealToStr(FMinVal,-1)+' <= '+s;
  if (ValueCheck=vcMax) or (ValueCheck=vcBoth)
    then s:=s+' <= '+RealToStr(FMaxVal,-1);
  if ValueCheck=vcNone then
    if Language=Russian then s:=s+' - .' else s:=s+' - any.';
  if ColorError<>ColorUsual then Hint:=s;
  if not Valid and not (csDesigning in ComponentState)
               and not (csLoading in ComponentState)
               and Visible and Enabled
               and Parent.Visible and Parent.Enabled then SetFocus;
end;


Procedure TINumEdit.SetIntCurVal(v:TInt);
begin  SetCurVal(1.0*v);  end;


//  -------------------- TAnimatedButton --------------------------

Constructor TAnimatedButton.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Height:=20; Width:=20; ShowHint:=true;
end;

Destructor TAnimatedButton.Destroy;
begin
  DeActivate;
  inherited Destroy;
end;

Procedure TAnimatedButton.CMVisibleChanged(var m:TMessage);
begin
  if Assigned(FTimer) and not (csDesigning in ComponentState)
    then FTimer.Enabled:=Self.Visible;
  inherited;
end;

Procedure TAnimatedButton.DefaultHandler(var Message);
begin
  Case TMessage(Message).Msg of
    WM_MOUSEMOVE: Application.Tag:=Self.HelpContext;
  end;
  inherited;
end;


Function TAnimatedButton.Activate(fn:string; freq:integer):boolean;
var i:integer;
    ImageList:TImageList; BM:TBitMap;
begin if freq<1 then freq:=20;               // default
  DeActivate;
  ImageList:=TImageList.Create(Self);
  Result:=ImageList.FileLoad(rtBitMap,fn,Color);
  if not Result then begin ImageList.Free; Exit; end;
  FBitMapList:=TList.Create;  BM:=TBitMap.Create; BM.Assign(Glyph); FBitMapList.Add(BM); Glyph:=nil;
  for i:=0 to ImageList.Count-1 do
    begin BM:=TBitMap.Create;  ImageList.GetBitMap(i,BM); FBitMapList.ADD(BM); end;
  TickTimer(Self);
  if ImageList.Count<2 then Exit;            // static glyph
  FTimer:=TTimer.Create(nil);  FTimer.Interval:=freq;
  FTimer.OnTimer:=TickTimer;   FTimer.Enabled:=Self.Visible;
end;

Procedure TAnimatedButton.Deactivate;
var i:integer; BM:TBitMap;
begin
  if Assigned(FTimer) then begin FTimer.Enabled:=false; FTimer.Free; FTimer:=nil; end;
  if Assigned(FBitMapList) then
  begin
    BM:=FBitMapList[0]; Glyph.Assign(BM);
    FCurrentImage:=0; TickTimer(Self);     // restore initial glyph
    for i:=0 to FBitMapList.Count-1 do begin BM:=FBitMapList[i]; BM.Free; end;
    FBitMapList.Clear;  FBitMapList.Free;  FBitMapList:=nil;
  end;
end;

Function TAnimatedButton.Active:boolean;
begin
  Result:=Assigned(FTimer);
  if Result then Result:=FTimer.Enabled;
end;

Procedure TAnimatedButton.TickTimer(Sender: TObject);
var DC1,DC2: THandle;
    BM:TBitMap;
begin
  if not Visible or not Assigned(FBitMapList) then Exit;
  BM:=FBitMapList[FCurrentImage];
  Inc(FCurrentImage); if FCurrentImage=FBitMapList.Count then FCurrentImage:=1;
  DC1 := GetDC(Parent.Handle);
  DC2:=CreateCompatibleDC(0);
  SelectObject(DC2,BM.Handle);
  BitBlt(DC1, Left+((Width-BM.Width) div 2), Top+((Height-BM.Height) div 2), BM.Width, BM.Height, DC2, 0, 0, SRCCOPY);
  ReleaseDC(BM.Handle,DC1);
  DeleteDC(DC2);
end;

end.
